@@ -1,5 +1,8 @@
+# frozen_string_literal: true
+
 require 'thread'
 require 'socket'
+require 'stringio'
 require 'timeout'
 
 module Riddle #:nodoc:
@@ -67,8 +70,11 @@ end
 
 require 'riddle/auto_version'
 require 'riddle/client'
+require 'riddle/command_failed_error'
+require 'riddle/command_result'
 require 'riddle/configuration'
 require 'riddle/controller'
+require 'riddle/execute_command'
 require 'riddle/query'
 
 Riddle.loaded_version = nil
