@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 class Riddle::AutoVersion
   def self.configure
     controller = Riddle::Controller.new nil, ''
@@ -10,7 +12,7 @@ class Riddle::AutoVersion
       require 'riddle/1.10'
     when /2.0.[12]/
       require 'riddle/2.0.1'
-    when /2.0.[^12]/, /2.1.\d/, /2.2.\d/
+    else
       require 'riddle/2.1.0'
     end
   end
